import _winreg

import sys

from os.path import abspath

import logging



#constants

EXE = abspath(sys.argv[0])



#constants for MIME RSS type support

MR_CLASSROOT = "rssfile"

MR_EXTENSION = ".rss"

MR_SUBKEY = r"%s\shell\open\command" % MR_CLASSROOT

MR_COMMAND = '"%s" --add-feed-from-rss "%%1"' % EXE

MR_MIME_TYPE = "application/rss+xml"



#constants for pseudo-protocol support

PP_PROTOCOL = "podcast"

PP_SUBKEY = r"%s\shell\open\command" % PP_PROTOCOL

PP_COMMAND = '"%s" --add-feed "%%1"' % EXE



#constants for MIME type support

log = logging.getLogger('iPodder')



def do_registrations(override=False):

    """Returns a two tuple containing (are_we_registered,was_anybody_registered)

    are_we_registered: True if iPodder is registered by the time we've finished.

    was_anybody_registered: True if us or another app was registered when we started."""

    if not is_exe():

        log.info("Oneclick: do_registrations returning True because we're not an executable")

        return (True,True)

    

    (mr_exists,mr_contains_me) = mime_rss_is_registered_to_me()

    if not mr_exists or (not mr_contains_me and override):

        log.info("Registering %s MIME type to iPodder." % MR_MIME_TYPE)

        register_mime_rss_to_me()



    was_anybody_registered = mr_exists

    

    if not override and (\

        (mr_exists and not mr_contains_me)):

        log.info("Some one-click behaviors not registered.")

        return (False,was_anybody_registered)

    else:

        return (True,was_anybody_registered)



def is_exe():

    return hasattr(sys,"frozen")



def check_exe():

    if not is_exe():

        log.exception("Running registry utils on a python file!  Careful how you interpret the results.")



def subkey_exists_and_contains_me(subkey):

    """Returns a 2-tuple of booleans: (exists,contains_me)"""

    check_exe()

    

    openkeys = []

    exists = False

    contains_me = False



    try:    

        try:

            key = _winreg.OpenKeyEx(_winreg.HKEY_CLASSES_ROOT,subkey,0,_winreg.KEY_QUERY_VALUE)

            openkeys.insert(0,key)

            exists = True

            info,type = _winreg.QueryValueEx(key, "")

            if info.find(EXE) != -1:

                contains_me = True

        finally:

            for key in openkeys:

                _winreg.CloseKey(key)

    except WindowsError, e:

        errno, message = e.args

        if errno != 2:

            raise e



    return (exists,contains_me)



def mime_check_file_extension():

    """Returns True if the MR_EXTENSION is mapped to the MR_CLASSROOT

    and False otherwise."""

    check_exe()

    

    openkeys = []

    result = False

    

    try:    

        try:

            key = _winreg.OpenKeyEx(_winreg.HKEY_CLASSES_ROOT,MR_EXTENSION,0,_winreg.KEY_QUERY_VALUE)

            openkeys.insert(0,key)

            exists = True

            info,type = _winreg.QueryValueEx(key, "")

            result = (info == MR_CLASSROOT)

        finally:

            for key in openkeys:

                _winreg.CloseKey(key)

    except WindowsError, e:

        errno, message = e.args

        if errno != 2:

            raise e



    return result



def mime_check_content_type():

    """Returns True if the MIME content type is mapped to the MR_EXTENSION

    and False otherwise."""

    check_exe()

    

    openkeys = []

    result = False

    subkey = r"MIME\Database\Content Type\%s" % MR_MIME_TYPE

    

    try:    

        try:

            key = _winreg.OpenKeyEx(_winreg.HKEY_CLASSES_ROOT,subkey,0,_winreg.KEY_QUERY_VALUE)

            openkeys.insert(0,key)

            exists = True

            info,type = _winreg.QueryValueEx(key, "Extension")

            result = (info == MR_EXTENSION)

        finally:

            for key in openkeys:

                _winreg.CloseKey(key)

    except WindowsError, e:

        errno, message = e.args

        if errno != 2:

            raise e



    return result

    

def mime_rss_is_registered_to_me():

    # Check the class root.

    (exists,contains_me) = subkey_exists_and_contains_me(MR_SUBKEY)

    if (not exists) or (not contains_me):

        return (exists,contains_me)



    # Check the MIME type.

    if not mime_check_content_type():

        return (exists,False)



    # Check the file extension.

    if not mime_check_file_extension():

        return (exists,False)

    

    # If we get here, we're registered!

    return (True,True)

    

def register_mime_rss_to_me():

    check_exe()



    openkeys = []

    result = False



    try:

        key1 = _winreg.CreateKey(_winreg.HKEY_CLASSES_ROOT,MR_CLASSROOT)

        openkeys.insert(0,key1)

        _winreg.SetValue(key1,"",_winreg.REG_SZ,"iPodder")



        key2 = _winreg.CreateKey(key1,"shell")

        openkeys.insert(0,key2)



        key3 = _winreg.CreateKey(key2,"open")

        openkeys.insert(0,key3)

        

        key4 = _winreg.CreateKey(key3,"command")

        openkeys.insert(0,key4)

        _winreg.SetValue(key4,"",_winreg.REG_SZ,MR_COMMAND)



        #File extension

        key5 = _winreg.CreateKey(_winreg.HKEY_CLASSES_ROOT,MR_EXTENSION)

        openkeys.insert(0,key5)

        _winreg.SetValue(key5,"",_winreg.REG_SZ,MR_CLASSROOT)



        #MIME TYPE

        key6 = _winreg.CreateKey(_winreg.HKEY_CLASSES_ROOT,"MIME")

        openkeys.insert(0,key6)

        key7 = _winreg.CreateKey(key6,"Database")

        openkeys.insert(0,key7)

        key8 = _winreg.CreateKey(key7,"Content Type")

        openkeys.insert(0,key8)

        key9 = _winreg.CreateKey(key8,MR_MIME_TYPE)

        openkeys.insert(0,key9)

        _winreg.SetValueEx(key9,"Extension",0,_winreg.REG_SZ,MR_EXTENSION)        

    finally:

        #assumes deepest keys come first.

        for key in openkeys:

            _winreg.CloseKey(key)



def pseudo_protocol_is_registered_to_me():

    return subkey_exists_and_contains_me(PP_SUBKEY)



def register_pseudo_protocol_to_me():

    check_exe()



    openkeys = []

    result = False



    try:

        key1 = _winreg.CreateKey(_winreg.HKEY_CLASSES_ROOT,PP_PROTOCOL)

        openkeys.insert(0,key1)

        _winreg.SetValue(key1,"",_winreg.REG_SZ,"URL:%s protocol" % PP_PROTOCOL)

        _winreg.SetValueEx(key1,"URL Protocol",0,_winreg.REG_SZ,"")



        key2 = _winreg.CreateKey(key1,"shell")

        openkeys.insert(0,key2)



        key3 = _winreg.CreateKey(key2,"open")

        openkeys.insert(0,key3)

        

        key4 = _winreg.CreateKey(key3,"command")

        openkeys.insert(0,key4)

        _winreg.SetValue(key4,"",_winreg.REG_SZ,PP_COMMAND)

    finally:

        #assumes deepest keys come first.

        for key in openkeys:

            _winreg.CloseKey(key)



if __name__ == "__main__":

    print "Checking content type...%s" % mime_check_content_type()

    print "Checking file extension...%s" % mime_check_file_extension()

